program example;
{ A simple example of using the BTREE unit }
uses crt,btree;
type

  Customer_Rec = record
                   Account : string[10];
                   Name    : string[30];
                   Tel     : string[20];
                 end;

var
   CustomerRec : Customer_Rec;
   Customer    : BIndex;
   IOError     : integer;

procedure Pad(Var s:string;MaxL:integer);
var i : integer;
begin
  if length(s)<MaxL then
    for i:=succ(length(s)) to MaxL do s:=s+' ';
end;
function OpenCustomerFile:boolean;
begin
{ Initialise the customer file object }

  Customer.Init;

{ Define key 1  Type                     String type
                Offset within the record 0
                Length of string         10
}

  Customer.DefineKey(1,B_STRING,0,10);

  IOError:=Customer.Open('Customer.idx','Customer.dta',sizeof(CustomerRec));
  if IOError=2 then                    { DOS error - data file not found }
    IOError:=Customer.Create('Customer.idx','Customer.dta',sizeof(CustomerRec))
  else
    if IOError=B_INDEX_NOT_FOUND then  { Index file not found }
      IOError:=Customer.Sort('Customer.idx','Customer.dta',sizeof(CustomerRec));;          
  if IOError=0 then
    OpenCustomerFile:=true
  else
    OpenCustomerFile:=false;
end;

procedure CloseCustomerFile;
begin
  IOError:=Customer.CloseFile;
end;

function GetAccount:boolean;
begin
  GetAccount:=false;
  clrscr;
  with CustomerRec do
  begin
    Write(' Enter Account ');
    readln(CustomerRec.Account);

{ The read method will build the key from the customer record, and
  search for that key in the index. If it is found, the record is returned in
  the customer record. If it not found the customer record is not changed. }

    IOError:=Customer.Read(CustomerRec);
    if IOError=0 then GetAccount:=true;
  end;
end;
procedure AddARecord;
begin
  if GetAccount then
    writeln('Duplicate Account')
  else
  begin
    writeln;
    write('Name ');
    readln(CustomerRec.Name);
    write('Tel  ');
    readln(CustomerRec.Tel);
    IOError:=Customer.Write(CustomerRec);
{ Builds the key from the customer record, adds the data record, and inserts
  the key into the index.
}
  end;
  writeln('Press Enter');
  readln;
end;
procedure DeleteARecord;
begin
  if GetAccount then
  begin
{ Builds the key from the customer record, searches the index, and, if found,
  deletes the data record and the index
}
    IOError:=Customer.DeleteRecord(CustomerRec);
  end
  else
    writeln('Account not found');
  writeln('Press Enter');
  readln;
end;

procedure EditARecord;
begin
  if GetAccount then
  begin
    writeln;
    write('Name ');
    readln(CustomerRec.Name);
    write('Tel  ');
    readln(CustomerRec.Tel);
{ Update thr current record }
    IOError:=Customer.RewriteRecord(CustomerRec);
  end
  else
    writeln('Account not found');
  writeln('Press Enter');
  readln;

end;

procedure ListRecords;
var Line : integer;
begin
  clrscr;
  writeln('BTREE Example');
  writeln('Account    Name                           Telephone');
  CustomerRec.Account:='';
  IOError:=Customer.Start(CustomerRec);
{ Returns the record whose key is not less than Account }
  line:=0;
  while IOError=0 do
  begin
    if Line>22 then
    begin
      Line:=0;
      readln;
    end;
    inc(Line);
    with CustomerRec do
    begin
      Pad(CustomerRec.Account,10);
      Pad(CustomerRec.Name,30);
      writeln(Account,' ',Name,' ',Tel);
    end;
    IOError:=Customer.Next(CustomerRec);
{ Returns the next record }
  end;
  writeln('Press Enter');
  readln;
end;

procedure EditSomeRecords;
var ch : char;
begin
  repeat
    clrscr;
    writeln('   BTREE example  ');
    gotoxy(1,24);
    write('(A)dd record   (D)elete Record  (E)dit Record   (L)ist Records');
    ch:=UPCASE(readkey);
    case ch of
      'A' : AddARecord;
      'D' : DeleteARecord;
      'E' : EditARecord;
      'L' : ListRecords;
    end;
  until ch=#27;
end;

begin
{ First, you must call InitialiseBTREE. This will allocate memmory on the
  heap used for manipulating key pages.
}
  if InitialiseBTREE then
  begin
    if OpenCustomerFile then
      EditSomeRecords;
    CloseCustomerFile;
  end;
end.